;;(setq w32-use-native-image-API t)
(require 'image)
(defun image-animate-timeout (image n count time-elapsed limit target-time)
  "Display animation frame N of IMAGE.
N=0 refers to the initial animation frame.
COUNT is the total number of frames in the animation.
TIME-ELAPSED is the total time that has elapsed since
`image-animate-start' was called.
LIMIT determines when to stop.  If t, loop forever.  If nil, stop
 after displaying the last animation frame.  Otherwise, stop
 after LIMIT seconds have elapsed.
The minimum delay between successive frames is `image-minimum-frame-delay'.

If the image has a non-nil :speed property, it acts as a multiplier
for the animation speed.  A negative value means to animate in reverse."
  (when (and (buffer-live-p (plist-get (cdr image) :animate-buffer))
             ;; Delayed more than two seconds more than expected.
             (or (time-less-p (time-since target-time) 2)
                 (progn
                   (message "Stopping animation; animation possibly too big")
                   nil)))
    (image-show-frame image n t)
    (let* ((speed (image-animate-get-speed image))
           (time (current-time))
           (animation (image-multi-frame-p image))
           (time-to-load-image (time-since time))
           (stated-delay-time (/ (or (cdr animation)
                                     image-default-frame-delay)
                                 (float (abs speed))))
           ;; Subtract off the time we took to load the image from the
           ;; stated delay time.
           (delay (max (float-time (time-subtract stated-delay-time
                                                  time-to-load-image))
                       image-minimum-frame-delay))
           done)
      (message "Time: %S, time-to-load-image: %S, stated-delay-time: %S, speed: %S"
               time time-to-load-image stated-delay-time speed)
      (setq n (if (< speed 0)
                  (1- n)
                (1+ n)))
      (if limit
          (cond ((>= n count) (setq n 0))
                ((< n 0) (setq n (1- count))))
        (and (or (>= n count) (< n 0)) (setq done t)))
      (setq time-elapsed (+ delay time-elapsed))
      (if (numberp limit)
          (setq done (>= time-elapsed limit)))
      (unless done
        (run-with-timer delay nil #'image-animate-timeout
                        image n count time-elapsed limit
                        (+ (float-time) delay))))))


(setq debug-on-error t)
(defvar test-image  (create-image "image-test-large.gif"))
(message "Delay: %S" (image-multi-frame-p test-image))
(insert-image test-image)
(pop-to-buffer "*Messages*")
(image-animate test-image)
